#!/usr/bin/env python
# -*- coding: utf-8 -*

"""
Утилита для аппаратной конфигурации оборудования (настройки тачскрина, портов)
"""
from datetime import datetime

import os
import shutil
import subprocess
import sys
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.dialog import dialog

serial_conf_file = '/etc/serial.conf'
conf_path = '/linuxcash/cash/tools/hardwareconf/conf/'
xorg_file = '/etc/X11/xorg.conf'
egalax_file = '/usr/lib/X11/xorg.conf.d/11-egalax.conf'
xfdrv_file = '/usr/lib/xorg/modules/input/xfdrvtouch_drv.so'
artix_file = '/usr/lib/X11/xorg.conf.d/artix.conf/'
check_x11_flag = '/linuxcash/cash/data/tmp/check-x11.flag'

isFirstrun = False
needToReboot = False

functions = {
    'no': lambda: no(),
    'default': lambda: default(),
    'flypos': lambda: flypos(),
    'bleep': lambda: bleep(),
    'checkway': lambda: checkway(),
    'shopathome': lambda: shopathome(),
    'cube': lambda: cube(),
    'fujitsu': lambda: fujitsu()
}

def menu(d):
    """
    Меню выбора способа настройки
    """
    d.add_persistent_args(("--backtitle", "Настройка аппаратной конфигурации"))
    return d.menu(
        text = "Настройка аппаратной конфигурации",
        width=40, 
        height=16,
        menu_height=9,
        choices=[
                 ("no", "Оставить без изменений"),
                 ("default", "По умолчанию"),    #4
                 ("flypos", "FlyPOS PRO PA53"),  #6
                 ("bleep", "Bleep TS-915"),      #6
                 ("checkway", "CheckWay"),       #6
                 ("cube", "Cube"),               #10
                 ("fujitsu", "Fujitsu"),         #10
                 ("shopathome", "Магазин у дома"),   #6
                 ("expansion", "Плата расширения")
                 ])

def set_settings_for(name):
    """
    Задать настройки для устройства с именем name
    """
    if name in functions:
        # Удаляем все конфигурационные файлы
        if name != "no":
            [ os.remove(f) for f in [xorg_file, egalax_file, xfdrv_file, artix_file] if os.path.exists(f) ]
            open(check_x11_flag, 'a').close()
        # Выполняем выбранный метод
        is_reboot = functions[name]()
        return True
    else:
        return False

def backup(filename):
    """
    Создать бэкап файла filename
    """
    current_daytime = datetime.now().strftime("%Y%m%d-%H%M")
    backup_filename = filename + "." + current_daytime
    shutil.move(filename, backup_filename)

def editGrub(grub_arg):

    with open('/etc/default/grub', 'r') as file:
        data = file.readlines()

    for a in range(len(data)):
        if 'GRUB_CMDLINE_LINUX_DEFAULT' in data[a]:
            data[a] = 'GRUB_CMDLINE_LINUX_DEFAULT="quiet splash %s"\n' % grub_arg

    with open('/etc/default/grub', 'w') as file:
        file.writelines( data )

    os.system("update-grub")

def setSerialConf(serial_conf_default_file):
    """
    Настройка файла /etc/serial.conf
    """
    backup(serial_conf_file)
    shutil.copy(serial_conf_default_file, serial_conf_file)

def no():
    """
    Выйти без изменений
    """
    exit_util(0)

def default():
    """
    Настроить по умолчанию
    """
    setSerialConf("/etc/serial.conf.default")
    shutil.copy('%s/default/xorg.conf' % (conf_path), xorg_file)
    editGrub('')

def flypos():
    global needToReboot
    needToReboot = True
    setSerialConf("/etc/serial.conf.default")
    shutil.copy('%s/flypos/xfdrvtouch_drv.so' % (conf_path), xfdrv_file)
    shutil.copy('%s/flypos/xorg.conf' % (conf_path), xorg_file)
    editGrub('8250.nr_uarts=6')

def bleep():
    setSerialConf("/etc/serial.conf.default")
    shutil.copy('%s/bleep/11-egalax.conf' % (conf_path), egalax_file)
    shutil.copy('%s/bleep/xorg.conf' % (conf_path), xorg_file)
    editGrub('8250.nr_uarts=6 i8042.noloop=1 usbhid.quirks=0xeef:0x1:0x40')

def checkway():
    setSerialConf("/etc/serial.conf.default")
    shutil.copy('%s/default/xorg.conf' % (conf_path), xorg_file)
    editGrub('8250.nr_uarts=6')

def shopathome():
    setSerialConf("/etc/serial.conf.default")
    shutil.copy('%s/default/xorg.conf' % (conf_path), xorg_file)
    editGrub('8250.nr_uarts=6')

def cube():
    """
    Настроить cube
    """
    setSerialConf("/etc/serial.conf.cube")
    shutil.copy('%s/default/xorg.conf' % (conf_path), xorg_file)
    editGrub('8250.nr_uarts=10')

def fujitsu():
    """
    Настроить fujitsu
    """
    setSerialConf("/etc/serial.conf.fujitsu")
    shutil.copy('%s/default/xorg.conf' % (conf_path), xorg_file)
    editGrub('8250.nr_uarts=10')

def expansion_card(n):
    """
    Настроить плату расширения
    """
    setSerialConf("/etc/serial.conf.default")
    shutil.copy('%s/default/xorg.conf' % (conf_path), xorg_file)
    editGrub('8250.nr_uarts=%s' % (n))

def exit_util(code):
    """
    Выйти из программы
    """
    subprocess.call("clear", shell=True)
    sys.exit(code)

def reboot(d):
    if isFirstrun and needToReboot:
        d.msgbox("Сейчас будет выполнена перезагрузка кассы, после чего настройка будет продолжена",
                 title="Необходима перезагрузка")
        subprocess.call("python /linuxcash/cash/tools/tools_avail/save_executed.py artix_hardwareconf", shell=True)
        subprocess.call("shutdown -r now", shell=True)
        subprocess.call("kill -9 `ps --pid %s -oppid=`" % os.getpid(), shell=True)
    else:
        d.msgbox("Изменения вступят в силу после перезагрузки")

def main():
    try:
        d = dialog.Dialog()
        (code, tag) = menu(d)
        if code == d.DIALOG_OK:
            try:
                if tag == "expansion":
                    (res, ports) = d.inputbox (
                            text = "Количество портов",
                            height = 7,
                            width = 40,
                            init = "4"
                            )
                    if res != d.DIALOG_OK:
                        exit_util(0)
                    nports = 0
                    try:
                        nports = int(ports)
                    except:
                        d.msgbox("Введены некорректные данные.")
                        exit_util(1)
                    expansion_card(nports)
                else:
                    result = set_settings_for(tag)
                    if not result:
                        exit_util(1)

                reboot(d)
                subprocess.call("/etc/init.d/etc-setserial restart", shell=True)
                exit_util(0)
            except IOError:
                d.msgbox("Невозможно изменить необходимые для настройки файлы", title="Ошибка")
                exit_util(1)
        else:
            exit_util(0)
    except KeyboardInterrupt:
        exit_util(0)

if __name__ == "__main__":
    global isFirstrun
    isFirstrun = len(sys.argv) > 1 and sys.argv[1] == 'firstrun'
    main()
